/****************************************************************************
* MeshLab                                                           o o     *
* A versatile mesh processing toolbox                             o     o   *
*                                                                _   O  _   *
* Copyright(C) 2005                                                \/)\/    *
* Visual Computing Lab                                            /\/|      *
* ISTI - Italian National Research Council                           |      *
*                                                                    \      *
* All rights reserved.                                                      *
*                                                                           *
* This program is free software; you can redistribute it and/or modify      *   
* it under the terms of the GNU General Public License as published by      *
* the Free Software Foundation; either version 2 of the License, or         *
* (at your option) any later version.                                       *
*                                                                           *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
* GNU General Public License (http://www.gnu.org/licenses/gpl.txt)          *
* for more details.                                                         *
*                                                                           *
****************************************************************************/
/****************************************************************************
  History
$Log: customDialog.cpp,v $
Revision 1.3  2005/12/04 16:50:15  glvertex
Removed [using namespace] directive form .h
Renaming in QT style
Adapted method behavior to the new ui interface

Revision 1.2  2005/12/03 22:49:46  cignoni
Added copyright info

****************************************************************************/

#include "customDialog.h"
#include <QPalette>

using namespace vcg;

CustomDialog::CustomDialog(QWidget * parent)
		:QDialog(parent)
{
	ui.setupUi(this);
	connect(ui.okButton, SIGNAL(clicked()), this, SLOT(accept()));
	connect(ui.cancelButton, SIGNAL(clicked()), this, SLOT(reject()));
	connect(ui.pushButtonBottomBg,SIGNAL(clicked()),this,SLOT(setBkgBottomColor()));
	connect(ui.pushButtonTopBg,SIGNAL(clicked()),this,SLOT(setBkgTopColor()));
	connect(ui.pushButtonLogArea,SIGNAL(clicked()),this,SLOT(setLogAreaColor()));
	setFixedSize(260,155);
}


void CustomDialog::loadCurrentSetting(const Color4b& bb,const Color4b& bt,const Color4b& l)
{
	bkgBottomColor=bb;
	bkgTopColor=bt;
 	logAreaColor=l;

	// Changes the palette in the labels
	QPalette pbb(QColor(bb.V(0),bb.V(1),bb.V(2)));
	QPalette pbt(QColor(bt.V(0),bt.V(1),bt.V(2)));
	QPalette pl(QColor(l.V(0),l.V(1),l.V(2)));
	
	ui.labelBottmBg->setPalette(pbb);
	ui.labelTopBg->setPalette(pbt);
	ui.labelLogArea->setPalette(pl);
}


void CustomDialog::setBkgBottomColor()
{
	QColor bb=QColorDialog::getColor(QColor(255,255,255,255),this);
	QPalette palette(bb);
	ui.labelBottmBg->setPalette(palette);
	bkgBottomColor.V(0)=bb.red();
	bkgBottomColor.V(1)=bb.green();
	bkgBottomColor.V(2)=bb.blue();
}

void CustomDialog::setBkgTopColor()
{
	QColor bt=QColorDialog::getColor(QColor(255,255,255,255),this);
	QPalette palette(bt);
	ui.labelTopBg->setPalette(palette);
	bkgTopColor.V(0)=bt.red();
	bkgTopColor.V(1)=bt.green();
	bkgTopColor.V(2)=bt.blue();
}

void CustomDialog::setLogAreaColor()
{
	QColor l=QColorDialog::getColor(QColor(255,255,255,255),this);
	QPalette palette(l);
	ui.labelLogArea->setPalette(palette);
	logAreaColor.V(0)=l.red();
	logAreaColor.V(1)=l.green();
	logAreaColor.V(2)=l.blue();
}
